// Generated by CoffeeScript 1.8.0
var APIObject, DialogObject, DocumentInfoObject, DocumentObject, OnObject, TerminalObject, UtilityObject, root,
  __indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; },
  __slice = [].slice;

root = typeof exports !== "undefined" && exports !== null ? exports : this;


/*
@class Dialog
@description The Dialog object - used to invoke dialogs
 */

DialogObject = (function() {
  function DialogObject() {
    this.callbacks = {};
  }


  /*
  	@method alert
  	@description Display an alert dialog
  	@param title|required|string|The main dialog title
  	@param information|required|string|Related information
  	@param callback|optional|string|The function to be called, when the dialog closes
  	@example $$Dialog.alert("Finished!","The operation has finished")
   */

  DialogObject.prototype.alert = function(title, information, callback) {
    var ind;
    if (callback == null) {
      callback = null;
    }
    if (callback != null) {
      if (callback instanceof String) {
        callback = "_plugin." + callback + "();";
      } else {
        if (__indexOf.call(this.callbacks, callback) < 0) {
          this.callbacks.push(callback);
        }
        ind = this.callbacks.indexOf(callback);
        callback = "_plugin.Dialog.callbacks['" + guid + "']();";
      }
    }
    return core.alertDialogWithTitle_information_callback_(title, information, callback);
  };


  /*
  	@method getInput
  	@description Show an input dialog and get user input
  	@param title|required|string|The title/prompt of the dialog
  	@param callback|required|string|The function to be called, with the input, when the dialog closes
  	@return null|
  	@example my_plugin = {
  	@example 	init: ()->
  	@example 		$$Dialog.getInput("Search","gotInput");
  	@example  
  	@example 	gotInput: (input)->
  	@example 		$$Terminal.echo("The input is: " + input);
  	@example }
   */

  DialogObject.prototype.getInput = function(title, callback) {
    var result;
    result = _plugin.addCallback(callback);
    core.input_callback_(title, result);
    return null;
  };

  return DialogObject;

})();


/*
@class Document
@description The current Document object
 */

DocumentObject = (function() {
  function DocumentObject() {}


  /*
  	@method getContent
  	@description Get current document's content
  	@return string|The document's contents
  	@example content = $$Document.getContent()
   */

  DocumentObject.prototype.getContent = function() {
    return pp.getContent();
  };


  /*
  	@method getLabel
  	@description Get current document's label, as it appears in Peppermint
  	@return string|The document's label
  	@example label = $$Document.getLabel()
   */

  DocumentObject.prototype.getLabel = function() {
    return core.getLabel();
  };


  /*
  	@method getInfo
  	@description Get DocumentInfo of current document
  	@return DocumentInfo|A <a href='#api-class-documentinfo'>DocumentInfo</a> object or <code>null</code>, if document is not stored to disk
  	@example documentInfo = $$Document.getInfo()
   */

  DocumentObject.prototype.getInfo = function() {
    var fullPath;
    fullPath = pp.getFullPath();
    if (fullPath === "") {
      return null;
    } else {
      return new DocumentInfoObject(fullPath);
    }
  };


  /*
  	@method getTemp
  	@description Get a temporary file for current document
  	@param extension|required|string|Desired extension for the file
  	@return DocumentInfo|A <a href='#api-class-documentonfo'>DocumentInfo</a> object with the details of the temporary file
  	@example tempInfo = $$Document.getTemp()
   */

  DocumentObject.prototype.getTemp = function(extension) {
    var filePath;
    filePath = core.getTempFile_withExtension_(this.getContent(), extension);
    return new DocumentInfoObject(filePath);
  };

  DocumentObject.prototype.write = function(path, contents) {
    return core.document_write_(path, contents);
  };

  return DocumentObject;

})();


/*
@class DocumentInfo
@description An object representing current document's details
@property path|string|The full path of the document
@property filename|string|The document's pure filename
@property folder|string|The document's pure folder name
@property extension|string|The document's file extension
 */

DocumentInfoObject = (function() {
  function DocumentInfoObject(filepath) {
    var extension, fileName, folderPath, re;
    fileName = filepath.replace(/^.*[\\\/]/, '');
    folderPath = filepath.replace(fileName, '');
    re = /(?:\.([^.]+))?$/;
    extension = re.exec(fileName)[1];
    this.path = filepath;
    this.filename = fileName;
    this.folder = folderPath;
    this.extension = extension;
  }

  return DocumentInfoObject;

})();


/*
@class On
@description The main Event-handling object
 */

OnObject = (function() {
  OnObject.pluginName;

  function OnObject() {}


  /*
  	@method documentChanged
  	@description Set callback to be execute, when document changes
  	@param callback|required|function|The callback to be executed, or null to remove handler
  	@return null|
  	@example init: ()->
  	@example    $$x = 1;
  	@example    $$On.documentChanged(()=>
  	@example        $$Terminal.echo("Document changed");
  	@example        $$Terminal.echo("x = " + $$x++);
  	@example    );
  	@example  
  	@example halt: ()->
  	@example    $$On.documentChanged(null);
   */

  OnObject.prototype.documentChanged = function(callback) {
    if (callback != null) {
      return window.__.documentChangedEvents[this.pluginName] = callback;
    } else {
      return delete window.__.documentChangedEvents[this.pluginName];
    }
  };


  /*
  	@method cursorChanged
  	@description Set callback to be execute, when document changes
  	@param callback|required|function|The callback to be executed, or null to remove handler
  	@return null|
  	@example $$On.cursorChanged(()=>
  	@example    # do something
  	@example )
   */

  OnObject.prototype.cursorChanged = function(callback) {
    if (callback != null) {
      return window.__.cursorChangedEvents[this.pluginName] = callback;
    } else {
      return delete window.__.cursorChangedEvents[this.pluginName];
    }
  };

  return OnObject;

})();


/*
@class Terminal
@description The main Terminal object - representing the console panel
 */

TerminalObject = (function() {
  function TerminalObject() {
    this.environment = {};
  }


  /*
  	@method show
  	@description Show the terminal panel, if not already open
  	@return null|
  	@example $$Terminal.show()
   */

  TerminalObject.prototype.show = function() {
    pp.showConsoleTab();
    return null;
  };


  /*
  	@method clear
  	@description Clear the terminal panel
  	@return null|
  	@example $$Terminal.clear()
   */

  TerminalObject.prototype.clear = function() {
    terminal.clear();
    return null;
  };


  /*
  	@method echo
  	@description Echoes given string to the terminal
  	@param message|required|string|The message to be displayed
  	@param attributes|optional|string|A style attribute string
  	@return null|
  	@example $$Terminal.echo("Hello world!")
   */

  TerminalObject.prototype.echo = function(message, attributes) {
    var msg;
    if (attributes == null) {
      attributes = null;
    }
    msg = message;
    if (attributes != null) {
      msg = "[[" + attributes + "]" + message + "]";
    }
    terminal.echo(msg);
    return null;
  };


  /*
  	@method exec
  	@description Execute command in terminal
  	@param command|required|string|Command to be executed
  	@param args|optional|string,...|Arguments to be pass
  	@return string|The execution result
  	@example $$Terminal.exec("php","version")
   */

  TerminalObject.prototype.exec = function() {
    var args, command;
    command = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
    if ((args != null) && args[0] instanceof Array) {
      args = args[0];
    }
    if (Object.keys(this.environment).length === 0) {
      return pp.exec([command].concat(args), false);
    } else {
      return pp.execWithEnv([command].concat(args), this.environment, false);
    }
  };

  TerminalObject.prototype.setEnvironment = function(variable, value) {
    return this.environment[variable] = value;
  };

  return TerminalObject;

})();


/*
@class Utility
@description The main Utility object - representing the Utility Panel on the right side of the Peppermint window
 */

UtilityObject = (function() {
  function UtilityObject() {}


  /*
  	@method load
  	@description Load html tool in the Utility View
  	@param title|required|string|The new title for the Utility View
  	@param path|required|string|The path to the .html file to be loaded
  	@return null|
  	@example $$Utility.load("My great tool", "index.html")
   */

  UtilityObject.prototype.load = function(title, path) {
    core.loadUtility_label_(path, title);
    return null;
  };


  /*
  	@method loadDocument
  	@description Load custom document in the Utility View
  	@param title|required|string|The new title for the Utility View
  	@param content|required|string|The script's contents
  	@param spec|required|string|The spec to be used for highlighting
  	@return null|
  	@example $$Utility.loadDocument("My PHP Script", "<?php echo 'done'; ?>", "php")
   */

  UtilityObject.prototype.loadDocument = function(title, content, spec) {
    return core.loadEditorUtility_label_mode_(content, title, spec);
  };

  return UtilityObject;

})();

APIObject = (function() {
  function APIObject() {
    this.documentChangedEvents = [];
    this.cursorChangedEvents = [];
    this.Dialog = new DialogObject;
    this.Document = new DocumentObject;
    this.Terminal = new TerminalObject;
    this.Utility = new UtilityObject;
  }

  APIObject.prototype.documentChanged = function() {
    var e, plugin, _ref, _results;
    _ref = this.documentChangedEvents;
    _results = [];
    for (plugin in _ref) {
      e = _ref[plugin];
      _results.push(e());
    }
    return _results;
  };

  APIObject.prototype.cursorChanged = function() {
    var e, plugin, _ref, _results;
    _ref = this.cursorChangedEvents;
    _results = [];
    for (plugin in _ref) {
      e = _ref[plugin];
      _results.push(e());
    }
    return _results;
  };

  APIObject.prototype.coffee = function(code, bare, exec) {
    var result;
    if (bare == null) {
      bare = false;
    }
    if (exec == null) {
      exec = false;
    }
    if (typeof code === 'object') {
      result = CoffeeScript.compile(eval(code)[0], {
        bare: bare
      });
    } else {
      result = CoffeeScript.compile(code, {
        bare: bare
      });
    }
    if (exec) {
      return eval(result);
    } else {
      return result;
    }
  };

  APIObject.prototype.haltRunningPlugin = function() {
    if ((window._plugin != null) && (window._plugin.halt != null)) {
      return window._plugin.halt();
    }
  };

  APIObject.prototype.initPlugin = function(pluginName, pluginPath) {
    this.haltRunningPlugin();
    if ((this[pluginName] != null) && (window._plugin != null) && pluginName === window._plugin.name) {
      __.Terminal.echo("Reusing");
      return window._plugin.init();
    } else {
      __.Terminal.echo("Creating");
      window._plugin = null;
      window._plugin = this[pluginName];
      window._plugin.callbacks = {};
      window._plugin.addCallback = function(callback) {
        var ind;
        ind = pp.uuid();
        this.callbacks["" + ind] = callback;
        return "window._plugin.callbacks['" + ind + "']('%@');";
      };
      window._plugin.name = pluginName;
      window._plugin.path = pluginPath;
      window._plugin.Dialog = window.__.Dialog;
      window._plugin.Document = window.__.Document;
      window._plugin.On = new OnObject(pluginName);
      window._plugin.Terminal = new TerminalObject;
      window._plugin.Utility = window.__.Utility;
      window._plugin.inspect = function() {
        var obj, objs, res, _i, _len;
        objs = 1 <= arguments.length ? __slice.call(arguments, 0) : [];
        res = "";
        for (_i = 0, _len = objs.length; _i < _len; _i++) {
          obj = objs[_i];
          res += window.inspectObject(obj);
        }
        return res;
      };
      window._plugin["import"] = function(path, execute) {
        if (execute == null) {
          execute = true;
        }
        if (execute) {
          return core.importjs_(path);
        } else {
          return core.file_(path);
        }
      };
      if (window._plugin.menu != null) {
        core.registerPluginMenu_(JSON.stringify(Object.keys(window._plugin.menu())));
      }
      return window._plugin.init();
    }
  };

  return APIObject;

})();

if (!root.__) {
  root.__ = new APIObject;
}

window.runPlugin = function(plContent, plName, plPath) {
  __.coffee(plContent, false, true);
  return __.initPlugin(plName, plPath);
};
